/*
Copyright 2012 Arun Kumar and Christopher Re

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This work was supported by the Microsoft Jim Gray Systems Lab, 
the National Science Foundation and the Office of Naval Research.
*/
#ifndef STACUTIL_H
#define STACUTIL_H
#include <iostream>
#include <cmath>
#include <fstream>
#include <vector>
#include <map>
#include <arpa/inet.h>
#include <cstring>
using namespace std;

class structureInfo {
public:
	map <int, vector <int> > stacgraph;	//underlying block graph structure
	vector<int> toplabs;								//this is an array indexed by the actual blknum
	vector<int> toplabsrev;							//this is an array indexed by the toplab and maps to the orig actual blknum
	vector<double> blkreachprobs;
	vector<double> blkprobs;						//both these are indexed by orig blknums;
	map <int, pair <int, int> > mmlens;	//all these are unused
	structureInfo(){}
	~structureInfo(){}
};

void incrsummatrs (double* dest, double *inp, int NUMST);
void multmatrs(double* dest, double* m1, double* m2, int NUMST);
void printmatr(double* m, int NUMST);
void scalarmultmatr (double* dest, int NUMST, double scalprob);
void matrdfs (double* probmatrs, double* scratches, int* memo, int NUMST, map <int, vector <int> > *stacgraph, int curnode);

void graphread(structureInfo *stacinfo, char* buffer);
void graphwrite(structureInfo *stacinfo, ostream &fout);

void reverseGraph(map <int, vector <int> > *gph, map <int, vector <int> > *revgph);
void sortArray(int* array, int arlen);
#endif
