/*
Copyright 2012 Arun Kumar and Christopher Re

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This work was supported by the Microsoft Jim Gray Systems Lab, 
the National Science Foundation and the Office of Naval Research.
*/
#include "indexing/indexgenUtils.h"

string toLower(string inp) {
	string ret = inp;
	for(int i=0; i<inp.length(); i++)
		if((ret[i] >= 65) && (ret[i] <= 90))			//assumed only ascii alphabet
			ret[i] = (char)((ret[i] - '\0') + 32);	//convert upper case alpha to lower case
	return ret;
}

//fwd dfs on trie; since it is a tree, no need to memoize; we can simply enum all paths -> each is a dict term
void fwdDFS(int *preftrie, map <int, string> *finals, int curst, string curstr) {
	map <int, string>::iterator tmp = (*finals).find(curst);
	if(tmp != (*finals).end())		//present; so, it is a final state
		(*finals)[curst] = curstr;	//mapping from finalstnum to dict term
	for(int j=0; j<NUMCHAR; j++) {
		if(preftrie[curst*NUMCHAR + j] == 0)
			continue;
		stringstream charstr;
		charstr << (char)(j + 32);
		string augstr = curstr + charstr.str();
		fwdDFS(preftrie, finals, preftrie[curst*NUMCHAR + j], augstr);
	}
}

//given the dfa in current set of stateoffs, read the char and move it to next set
inline set < pair <int, int> > getNextStateoffs(set < pair <int, int> > *stateoffs, int *preftrie, char ch) {
	set < pair <int, int> > retnext;
	for(set < pair <int, int> >::iterator iter = stateoffs->begin(); iter != stateoffs->end(); iter++) {
		int nxt = preftrie[(iter->first)*NUMCHAR + ((int)ch - 32)];
		if(nxt != 0)
			retnext.insert(make_pair(nxt, iter->second));
	}
	return retnext;
}

//this function walks the dfa thro the blocks
//we also track the visitcnt of a memo'd blk -> so that wn all children hv finished access to this blk, we can remove it from mem! this prevents mem from overshooting
//if numchildren is 0 or 1, we do NOT need to memoize; so, wn returning memo'd blks, we check visitcnt, and if it is equal to outdegree, we can delete it frmo the memo
augstates recEmit(map <int, vector <string> > *allpaths, map <int, vector <int> > *revgph, int *preftrie, map <int, string> *finals,  int thisblk, augstates *memoaugstates, int *memoflag, augstates *emissions, int *numchildren, int *visitcount) {
	visitcount[thisblk] += 1;
	if((memoflag)[thisblk] != -1)  {
		if(visitcount[thisblk] == numchildren[thisblk]) {
			augstates retval = (memoaugstates)[thisblk];
			(memoaugstates)[thisblk].clear();
			return retval;
		}
		else
			return (memoaugstates)[thisblk];
	}
	augstates allparents;		//this map of augstates unites the augstates of each parent; this way, we can avoid redundant runs per state
	for(vector <int>::iterator iter = (*revgph)[thisblk].begin(); iter != (*revgph)[thisblk].end(); iter++) {
		augstates tmp = recEmit(allpaths, revgph, preftrie, finals, *iter, memoaugstates, memoflag, emissions, numchildren, visitcount);
		for(map <int, set <location> >::iterator siter = tmp.begin(); siter != tmp.end(); siter ++) 		//unify with allparents
			for(set <location>::iterator liter = siter->second.begin(); liter != siter->second.end(); liter ++)
				allparents[siter->first].insert(*liter);
	}
	
	int tbnumpaths = (*allpaths)[thisblk].size();
	augstates nextaugs;	//final set of aug states to be passed to child
	for(int i = 0; i < tbnumpaths; i++) {
		string thispath = (*allpaths)[thisblk][i];
		int thislen = thispath.length();
		//start from state 0, and run the trie; this run will account for substr matching too; do this for each path
		set < pair <int, int> > stateoffs;	//set of current states of DFA adorned with offset (i if no match; if match, off doest increment)
		stateoffs.insert(make_pair(0, 0));	//initlze to 0,0;
		for(int j=0; j<thislen; j++) {
			set < pair <int, int> > nextstateoffs = getNextStateoffs(&stateoffs, preftrie, thispath[j]);
			//check for any final states, if so add emissions
			for(set < pair <int, int> >::iterator iter = nextstateoffs.begin(); iter != nextstateoffs.end(); iter++) {
				map <int, string>::iterator chk = finals->find(iter->first);
				if(chk != finals->end()) //emit, with adornment of this lineno, blkno
					(*emissions)[chk->first].insert(make_pair(thisblk, make_pair(i, iter->second)));
			}
			//add the mandatory 0 state with joff
			nextstateoffs.insert(make_pair(0, j+1));	//0 with joff is inserted
			stateoffs = nextstateoffs;	//exchange, and itereate over string
		}
		//we are done reading string; now, check stateoffs - if only 0 remains, ignore. else, add stateoffs with thisblk,lineno to nextaugs to pass on to child
		for(set < pair <int, int> >::iterator piter = stateoffs.begin(); piter != stateoffs.end(); piter++) {
			if(piter->first == 0)
				continue;
			nextaugs[piter->first].insert(make_pair(thisblk, make_pair(i, piter->second)));
		}
		//now, start from each of the par's augstate and do the above as for 0 (run the dfa)
		for(map <int, set <location> >::iterator aiter = allparents.begin(); aiter != allparents.end(); aiter++) {
			int curstate = aiter->first;	//current states of DFA -> no need to adorn this, since we inherit the adornment
			for(int j=0; j<thislen; j++) {
				//get next state, if any
				int nxt = preftrie[curstate*NUMCHAR + ((int)thispath[j] - 32)];
				if(nxt != 0) {
					curstate = nxt;
					//check if it is a final state, if so add emission
					map <int, string>::iterator chk = finals->find(curstate);
					if(chk != finals->end())	//emit, with adornment that was inherited. not this blk/line
						//here, we hv to iterate thro the adornmt locns for this finalst, and insert it into the emissions
						for(set <location>::iterator lociter = aiter->second.begin(); lociter != aiter->second.end(); lociter++)
							(*emissions)[chk->first].insert(*lociter);
				}
				else //else this wing simply dies; so ignore this string further, since substr have been consdiered before with 0 state
					break;
				if(j == (thislen - 1)) //the last char has been read; so, now if we have a valid state to pass on, add to nextaug
					for(set <location>::iterator lociter = aiter->second.begin(); lociter != aiter->second.end(); lociter++)
						nextaugs[curstate].insert(*lociter);	//unite the lcoatsiotn set and pass on
			}
		}//done the case for inh from pars
	}//done for over all paths
	//now, memoize and return; we need to memoize only if numchild > 1, since otherwise we will NOT visit this blk again
	if(numchildren[thisblk] > 1) {
		(memoaugstates)[thisblk] = nextaugs;
		(memoflag)[thisblk] = 1;	//we need a sep memo to see whether we have done this before; we cant depend on count of nihtoks since it maybe 0
	}
	return nextaugs;
}
