/*
Copyright 2012 Arun Kumar and Christopher Re

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This work was supported by the Microsoft Jim Gray Systems Lab, 
the National Science Foundation and the Office of Naval Research.
*/
#include <iostream>
#include <fstream>
#include <string>
#include <cstring>
#include <sstream>
#include <vector>
#include <cstdlib>
#include "libpq-fe.h"
using namespace std;

int main(int argc, char** argv) {
	if(argc < 7) {
		cout << "Usage: loadstaccato <dbname> <port number> <staccatodata table> <docname> <mbinname> <.stac file>\n";
		return 0;
	}
	PGconn* conn;
	string dbstring = "dbname=" +  string(argv[1]) + " port=" + string(argv[2]);
	conn = PQconnectdb(dbstring.c_str());
	if(PQstatus(conn) != CONNECTION_OK) {
		cout << "Error: Could not connect with " << dbstring << "! Message: " << PQerrorMessage(conn);
		PQfinish(conn);
		exit(1);
	}
	string stactablename = string(argv[3]);
	string docname = string(argv[4]);
	string fstname = string(argv[5]);
	ifstream fin(argv[6], ios::in);
	string pathkey = docname + "-" + fstname;
	int M, K, numblks;
	string line1, line2;
	getline(fin, line1);
	getline(fin, line2);
	stringstream line1str(line1);
	stringstream line2str(line2);
	stringstream Mstr, Kstr;
	line1str >> M >> K;
	line2str >> numblks;
	Mstr << M;
	Kstr << K;		
	string gphline;
	for(int i=0; i<numblks; i++) //ignore the stacgraph line
		getline(fin, gphline);
	vector <string> tuples;
	for(int b=0; b<numblks; b++) {
		int blknum, tmp, numpaths;
		string linet;
		getline(fin, linet);
		stringstream linetstr(linet);
		linetstr >> blknum >> tmp >> numpaths;	//the blknum need not be b
		for(int p=0; p<numpaths; p++) { //the pathnum is p
			string path, pathwt;
			getline(fin, path);
			getline(fin, pathwt);
			stringstream blknumstr;
			stringstream pstr;
			blknumstr << blknum;
			pstr << p;
			int slen = path.length();
			char tostr[2*slen+1];
			memset(tostr, '\0', 2*slen+1);
			int erro = 0;
			PQescapeStringConn(conn, tostr, path.c_str(), slen, &erro);
			if(erro != 0) {
				cout << "Error! Cannot escape string " << path << " and thus not inserting!" << endl;
				exit(1);
			}
			string thistuple = blknumstr.str() + ", " + pstr.str() + ", E'" + string(tostr) + "', " + pathwt;
			tuples.push_back(thistuple);
		}
	}
	string comndpaths = "insert into " + stactablename + " values ";	
	comndpaths += "('" + pathkey + "', " + tuples[0] + ")";
	for(int t=1; t<tuples.size(); t++) {
		comndpaths += ", ('" + pathkey + "', " + tuples[t] + ")";
	}
	comndpaths += ";";
	PGresult *res;
	res = PQexec(conn, comndpaths.c_str());
	PQclear(res);
	PQfinish(conn);	
return 0;
}
